

#include "Main.h"

#include "DXUtil.h"


// Matrix for DX operations
//D3DXMATRIX mDXMatrix;
#define DXSTACKSIZE 40
D3DXMATRIX mDXMatrixStack[DXSTACKSIZE];
int mDXStackPointer=0;



bool BuildVertexBuffer(IDirect3DVertexBuffer8 **pNewBuff, strVertex *pVertexData, int iVertexCount)
{

	HRESULT hr;
	unsigned char *vb_vertices;

	//Create a vertex buffer to hold the data
	hr=gD3DDevice->CreateVertexBuffer(iVertexCount * sizeof(strVertex), //Size of memory to be allocated
								   D3DUSAGE_WRITEONLY,  //Specify write only, it's faster
								   D3D8T_CUSTOMVERTEX,  //Our custom vertex specifier (coordinates & a colour)
								   D3DPOOL_MANAGED,     //Tell DirectX to manage the memory of this resource
								   pNewBuff);              //Pointer to our Vertex Buffer, after this call
														//It will point to a valid vertex buffer
	if(FAILED(hr))
	{
		OutputDebugString("Error creating vertex buffer (BuildVertexBuffer)");
		exit(5);
	}


	if (pVertexData != NULL)
	{
		//Now we have our Vertex Buffers, but they're empty. To put our data into them
		//we Lock the Vertex Buffer so Direct3D knows we're modifying it, then we copy
		//our data in and Unlock it so Direct3D knows we're done.
		hr=(*pNewBuff)->Lock(0, //Offset, we want to start at the beginning
						 0, //SizeToLock, 0 means lock the whole thing
						 &vb_vertices, //If successful, this will point to the data in the VB
						 0);  //Flags, nothing special
		if(FAILED(hr))
		{
			OutputDebugString("Error Locking vertex buffer (BuildVertexBuffer)");
			exit(5);
		}

		//vb_vertices now points to our vertices inside the Vertex buffer, so
		//to fill in our VB, we copy to vb_vertices.
		memcpy(vb_vertices, pVertexData, iVertexCount * sizeof(strVertex) );

		//Unlock so Direct3D knows we're done and can do any behind-the-scenes magic required
		(*pNewBuff)->Unlock();
	}


	return true;
}


void ScaleVertexBuffer(IDirect3DVertexBuffer8 *pVertexBuff, float fScaleFactor, int iVertexCount)
{
	int i;
	strVertex *pVertexData;
	HRESULT hr;

	// Lock the vertex buffer
	hr=pVertexBuff->Lock(0, //Offset, we want to start at the beginning
					 0, //SizeToLock, 0 means lock the whole thing
					 (unsigned char**)&pVertexData, //If successful, this will point to the data in the VB
					 0);  //Flags, nothing special
	if(FAILED(hr))
	{
		OutputDebugString("Error Locking vertex buffer (ScaleVertexBuffer)");
		exit(5);
	}

	for (i=0; i<iVertexCount; i++)
	{
		pVertexData->fX *= fScaleFactor;
		pVertexData->fY *= fScaleFactor;
		pVertexData->fZ *= fScaleFactor;

		pVertexData += 1;
	}

	// Unlock the buffer
	pVertexBuff->Unlock();
}



bool BuildIndexBuffer(IDirect3DIndexBuffer8 **pNewBuff, short *pIndexData, int iIndexCount)
{

	HRESULT hr;
	unsigned char *ib_indices;

	//Create an index buffer to hold the data
	hr=gD3DDevice->CreateIndexBuffer(iIndexCount * sizeof(short), //Size of memory to be allocated
								   D3DUSAGE_WRITEONLY,  //Specify write only, it's faster
								   D3DFMT_INDEX16,		//Buffer format
								   D3DPOOL_MANAGED,     //Tell DirectX to manage the memory of this resource
								   pNewBuff);           //Pointer to our Vertex Buffer, after this call
														//It will point to a valid vertex buffer
	if(FAILED(hr))
	{
		OutputDebugString("Error creating index buffer (BuildIndexBuffer)");
		exit(5);
	}


	if (pIndexData != NULL)
	{
		//Now we have our Index Buffer, but it's empty. To put our data into them
		//we Lock the Index Buffer so Direct3D knows we're modifying it, then we copy
		//our data in and Unlock it so Direct3D knows we're done.
		hr=(*pNewBuff)->Lock(0, //Offset, we want to start at the beginning
						 0, //SizeToLock, 0 means lock the whole thing
						 &ib_indices, //If successful, this will point to the data in the VB
						 0);  //Flags, nothing special
		if(FAILED(hr))
		{
			OutputDebugString("Error Locking index buffer (BuildIndexBuffer)");
			exit(5);
		}

		//ib_indices now points to our indices inside the Index buffer, so
		//to fill in our IB, we copy to ib_indices.
		memcpy(ib_indices, pIndexData, iIndexCount * sizeof(short) );

		//Unlock so Direct3D knows we're done and can do any behind-the-scenes magic required
		(*pNewBuff)->Unlock();
	}


	return true;
}

// DX functions


void dxApplyMatrix(IDirect3DDevice8 *D3DDevice)
{
	// Set the transformation matrix for the device
	D3DDevice->SetTransform(D3DTS_WORLD, &mDXMatrixStack[mDXStackPointer]);
}


void dxLoadIdentity(void)
{
	// Load an identity matrix to the current matrix stack element
	D3DXMatrixIdentity(&mDXMatrixStack[mDXStackPointer]);
}


void dxTranslatef(float fX, float fY, float fZ)
{
	D3DXMATRIX matrixTemp;

	// Multiply the current matrix by the required translation matrix
	D3DXMatrixMultiply(&mDXMatrixStack[mDXStackPointer], D3DXMatrixTranslation(&matrixTemp, fX, fY, fZ), &mDXMatrixStack[mDXStackPointer]);
}


void dxRotatef(float fAngle, float fX, float fY, float fZ)
{
	D3DXMATRIX matrixTemp;

	// Vector representing the axis of rotation
	D3DXVECTOR3 rotMatrix(fX, fY, fZ);

	// Scale the angle from degrees to radians
	fAngle *= DEG2RAD;

	D3DXMatrixMultiply(&mDXMatrixStack[mDXStackPointer], D3DXMatrixRotationAxis(&matrixTemp, &rotMatrix, fAngle), &mDXMatrixStack[mDXStackPointer]);
}

void dxScalef(float fX, float fY, float fZ)
{
	D3DXMATRIX matrixTemp;

	// Scale the current matrix
	D3DXMatrixMultiply(&mDXMatrixStack[mDXStackPointer], D3DXMatrixScaling(&matrixTemp, fX, fY, fZ), &mDXMatrixStack[mDXStackPointer]);
}


bool dxPushMatrix(void)
{
	// Is there any room left on the stack?
	if (mDXStackPointer < DXSTACKSIZE-1)
	{
		// Yes, so increase the stack pointer
		mDXStackPointer += 1;

		// Copy the previous matrix to this stack position
		mDXMatrixStack[mDXStackPointer] = mDXMatrixStack[mDXStackPointer-1];

		// All ok
		return true;
	}

	// No space on stack
	return false;
}

bool dxPopMatrix(void)
{
	// Is there anything remaining on the stack?
	if (mDXStackPointer > 0)
	{
		// Yes, so decrease the stack pointer
		mDXStackPointer -= 1;

		// All ok
		return true;
	}

	// Nothing left on stack
	return false;
}






// Reduces a normal vector specified as a set of three coordinates,
// to a unit normal vector of length 1.
void ReduceToUnit(float vector[3])
{
	float length;

	// Calculate the length of the vector
	length = (float)sqrt((vector[0]*vector[0]) +
	(vector[1]*vector[1]) +
	(vector[2]*vector[2]));

	// Keep the program from blowing up by providing an acceptable
	// value for vectors whose length may be calculated too close to zero.
	if(length == 0.0f)
	length = 1.0f;

	// Dividing each element by the length will result in a
	// unit normal vector.
	vector[0] /= length;
	vector[1] /= length;
	vector[2] /= length;
}


// Points p1, p2, & p3 specified in counterclockwise order
void CalcNormal(float v[3][3], float out[3])
{
	float v1[3],v2[3];
	static const int x = 0;
	static const int y = 1;
	static const int z = 2;

	// Calculate two vectors from the three points
	v1[x] = v[0][x] - v[1][x];
	v1[y] = v[0][y] - v[1][y];
	v1[z] = v[0][z] - v[1][z];

	v2[x] = v[1][x] - v[2][x];
	v2[y] = v[1][y] - v[2][y];
	v2[z] = v[1][z] - v[2][z];

	// Take the cross product of the two vectors to get
	// the normal vector which will be stored in out[]
	out[x] = v1[y]*v2[z] - v1[z]*v2[y];
	out[y] = v1[z]*v2[x] - v1[x]*v2[z];
	out[z] = v1[x]*v2[y] - v1[y]*v2[x];

	// Normalize the vector (shorten length to one)
	ReduceToUnit(out);
}
